package com.linkv.rtm_flutter_plugin;


import android.app.Application;
import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.text.TextUtils;
import android.util.Log;

import androidx.annotation.NonNull;

import com.im.imcore.IMBridger;
import com.im.imlogic.IMMsg;
import com.im.imlogic.LVIMSDK;
import com.im.imlogic.LVIMSession;
import com.im.imlogic.LVPushContent;
import com.linkv.lvrtmsdk.LVRTMEngine;
import com.linkv.rtcsdk.LinkVRTCEngine;
import com.linkv.rtcsdk.bean.VideoQuality;
import com.linkv.rtcsdk.utils.LogUtils;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

import static com.linkv.lvrtmsdk.LVRTMEngine.MessageSubType.IM_SUBTYPE_AUDIO;
import static com.linkv.lvrtmsdk.LVRTMEngine.MessageSubType.IM_SUBTYPE_IMAGE;
import static com.linkv.lvrtmsdk.LVRTMEngine.MessageSubType.IM_SUBTYPE_TEXT;
import static com.linkv.lvrtmsdk.LVRTMEngine.MessageSubType.IM_SUBTYPE_VIDEO;
import static com.linkv.rtcsdk.LinkVRTCEngine.VideoConfigLevel.VIDEO_CONFIG_1080P;
import static com.linkv.rtcsdk.LinkVRTCEngine.VideoConfigLevel.VIDEO_CONFIG_180P;
import static com.linkv.rtcsdk.LinkVRTCEngine.VideoConfigLevel.VIDEO_CONFIG_270P;
import static com.linkv.rtcsdk.LinkVRTCEngine.VideoConfigLevel.VIDEO_CONFIG_360P;
import static com.linkv.rtcsdk.LinkVRTCEngine.VideoConfigLevel.VIDEO_CONFIG_540P;
import static com.linkv.rtcsdk.LinkVRTCEngine.VideoConfigLevel.VIDEO_CONFIG_720P;

/**
 * RtmFlutterPlugin
 */
public class RtmFlutterPlugin implements FlutterPlugin, MethodCallHandler {
    private static ViewEventListener mViewEventListener;
    private MethodChannel channel;
    static LVRTMEngine mRtmEngine;
    private String TAG = "RtmFlutterPlugin";
    private Context mContext;
    private boolean isTokenRequesting;
    Handler handler = new Handler(Looper.getMainLooper());

    private void runOnUiThread(Runnable runnable) {
        handler.post(runnable);
    }

    public RtmFlutterPlugin() {
    }

    @Override
    public void onAttachedToEngine(FlutterPluginBinding flutterPluginBinding) {
        LogUtils.d(TAG, "onAttachedToEngine");
        mContext = flutterPluginBinding.getApplicationContext();
        channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "rtm_flutter_plugin");
        channel.setMethodCallHandler(this);
        flutterPluginBinding.getPlatformViewRegistry().registerViewFactory("surface", new FlutterSurfaceViewFactory(flutterPluginBinding.getBinaryMessenger()));
    }


    @Override
    public void onMethodCall(MethodCall call, @NonNull Result result) {
        LogUtils.d(TAG, "onMethodCall  : " + call.method);
        if (call.method.equals("getSdkVersion")) {
            result.success(mRtmEngine.getSdkVersion());
        } else if (call.method.equals("initSDK")) {
            initSDK(call, result);
        } else if (call.method.equals("loginUser")) {
            loginUser(call, result);
        } else if (call.method.equals("requestDebugToken")) {
            requestDebugToken(call, result);
        } else if (call.method.equals("setIMToken")) {
            setIMToken(call, result);
        } else if (call.method.equals("setLogOpen")) {
            setLogOpen(call);
        } else if (call.method.equals("sendPrivateMessage")) {
            sendPrivateMessage(call, result);
        } else if (call.method.equals("sendRoomMessage")) {
            sendRoomMessage(call, result);
        } else if (call.method.equals("uploadPushToken")) {
            uploadPushToken(call, result);
        } else if (call.method.equals("unInitSDK")) {
            unInitSDK(call, result);
        } else if (call.method.equals("loginRoom")) {
            loginRoom(call, result);
        } else if (call.method.equals("logoutRoom")) {
            logoutRoom(result);
        } else if (call.method.equals("logoutUser")) {
            logoutUser(result);
        } else if (call.method.equals("setPublishQualityMonitorCycle")) {
            setPublishQualityMonitorCycle(call, result);
        } else if (call.method.equals("startPublishing")) {
            startPublishing(call, result);
        } else if (call.method.equals("stopPublishStream")) {
            stopPublishStream(result);
        } else if (call.method.equals("startPlayingStream")) {
            startPlayingStream(call, result);
        } else if (call.method.equals("stopPlayingStream")) {
            stopPlayingStream(call, result);
        } else if (call.method.equals("setPlayQualityMonitor")) {
            setPlayQualityMonitor(call, result);
        } else if (call.method.equals("switchCamera")) {
            switchCamera(call);
        } else if (call.method.equals("setAVConfig")) {
            setAVConfig(call, result);
        } else if (call.method.equals("setMuteOutput")) {
            setMuteOutput(call);
        } else if (call.method.equals("startCapture")) {
            startCapture(result);
        } else if (call.method.equals("stopCapture")) {
            stopCapture(result);
        } else if (call.method.equals("isIMAuthed")) {
            isIMAuthed(result);
        } else if (call.method.equals("isIMLoginSucceed")) {
            isIMLoginSucceed(result);
        } else if (call.method.equals("setBeautyLevel")) {
            setBeautyLevel(call, result);
        } else if (call.method.equals("sendMessage")) {
            sendMessage(call, result);
        } else if (call.method.equals("sendEventMessage")) {
            sendEventMessage(call, result);
        } else if (call.method.equals("deleteLocalPrivateSession")) {
            deleteLocalPrivateSession(call, result);
        } else if (call.method.equals("clearPrivateSessionUnreadMsg")) {
            clearPrivateSessionUnreadMsg(call, result);
        } else if (call.method.equals("clearPrivateAllUnreadMsg")) {
            clearPrivateAllUnreadMsg(call, result);
        } else if (call.method.equals("updatePrivateMsgAsReadByStime")) {
            updatePrivateMsgAsReadByStime(call, result);
        } else if (call.method.equals("queryLocalPrivateHistoryMessage")) {
            queryLocalPrivateHistoryMessage(call, result);
        } else if (call.method.equals("querySessionList")) {
            querySessionList(call, result);
        } else if (call.method.equals("setPrivateDBStorageMax")) {
            setPrivateDBStorageMax(call,result);
        } else if (call.method.equals("queryUnReadMessageNum")) {
            queryUnReadMessageNum(call,result);
        } else {
            result.notImplemented();
        }
    }

    private void setPrivateDBStorageMax(MethodCall call, Result result) {
        int max = call.argument("max");
        if (max <= 0 ){
            result.success(-1);
            return;
        }
        int maxResult = LVIMSDK.sharedInstance().setPrivateDBStorageMax(max);
        result.success(maxResult);
    }

    private void queryUnReadMessageNum(MethodCall call, Result result) {
        int unReadNum = LVIMSDK.sharedInstance().queryUnReadMessageNum();
        result.success(unReadNum);
    }

    private void sendMessage(MethodCall call, Result result) {
        int cmdType = call.argument("cmdType");
        String fromID = call.argument("fromID");
        int sub = call.argument("sub");
        String tid = call.argument("tid");
        String type = call.argument("type");
        String content = call.argument("content");
        long cmsgid = call.argument("cmsgid");
        String pushContent = call.argument("pushContent");

        IMMsg msg = new IMMsg();
        msg.cmdType = cmdType;
        msg.fromID = fromID;
        msg.subType = sub;
        msg.toID = tid;
        msg.setMsgType(type);
        msg.setMsgContent(content);
        msg.setPushContent(pushContent);
        msg.cmsgid = cmsgid;

        LVIMSDK.sharedInstance().sendMessage(msg, null, new IMBridger.IMSendMessageListener() {
            @Override
            public void onIMSendMessageCallback(int ecode, String tid, IMMsg msg, Object o) {
                Map<String, Object> messageResultMap = convertMsgToMap(msg);
                messageResultMap.put("eCode", ecode);
                result.success(messageResultMap);
            }
        });
    }

    private void clearPrivateSessionUnreadMsg(MethodCall call, Result result) {
        String uid = call.argument("uid");
        int num = LVIMSDK.sharedInstance().clearPrivateSessionUnreadMsg(uid);
        result.success(num);
    }

    /// 获取消息会话列表
    private void querySessionList(MethodCall call, Result result) {
        List<LVIMSession> list = new ArrayList<>();
        Map<String, Object> resultMap = new HashMap<>();
        int num = LVIMSDK.sharedInstance().querySessionList(list);
        resultMap.put("result", num);

        List<Map<String, Object>> resultList = new ArrayList<>();
        for (LVIMSession session : list) {
            Map<String, Object> messageResultMap = convertMsgToMap(session.getLastMsg());
            messageResultMap.put("unreadCount", session.getUnreadCount());
            resultList.add(messageResultMap);
        }
        resultMap.put("sessions", resultList);
        result.success(resultMap);
    }


    private void queryLocalPrivateHistoryMessage(MethodCall call, Result result) {
        Map<String, Object> resultMap = new HashMap<>();
        ArrayList<IMMsg> msgList = new ArrayList<>();
        String uid = call.argument("uid");
        int dbid = call.argument("dbid");
        int limit = call.argument("limit");
        dbid = dbid == 0 ? Integer.MAX_VALUE : dbid;

        int num = LVIMSDK.sharedInstance().queryLocalPrivateHistoryMessage(uid, dbid, limit, true, msgList);
        resultMap.put("result", num);

        List<Map<String, Object>> resultList = new ArrayList<>();
        for (IMMsg msg : msgList) {
            Map<String, Object> messageResultMap = convertMsgToMap(msg);
            resultList.add(messageResultMap);
        }
        resultMap.put("msgs", resultList);
        result.success(resultMap);
    }


    private void updatePrivateMsgAsReadByStime(MethodCall call, Result result) {
        String uid = call.argument("uid");
        long stime = call.argument("stime");
        int num = LVIMSDK.sharedInstance().updatePrivateMsgAsReadByStime(uid, stime);
        result.success(num);
    }


    private void clearPrivateAllUnreadMsg(MethodCall call, Result result) {
        int num = LVIMSDK.sharedInstance().clearPrivateAllUnreadMsg();
        result.success(num);
    }

    private void deleteLocalPrivateSession(MethodCall call, Result result) {
        String uid = call.argument("uid");
        int num = LVIMSDK.sharedInstance().deleteLocalPrivateHistoryMessage(uid);
        result.success(num);
    }

    private void sendEventMessage(MethodCall call, Result result) {

        String tid = call.argument("tid");
        String type = call.argument("type");
        String content = call.argument("content");
        String  pushTitle = call.argument("pushTitle");
        String  pushBody = call.argument("pushBody");
        String  pushExtra = call.argument("pushExtra");
        String  pushClickAction = call.argument("pushClickAction");
        LVPushContent lvPushContent = new LVPushContent();
        lvPushContent.setTitle(pushTitle);
        lvPushContent.setBody(pushBody);
        lvPushContent.setExtra(pushExtra);
        lvPushContent.setClick_action(pushClickAction);
        IMMsg msg = IMMsg.buildEventMessage(tid, type, content, lvPushContent, null, null, null, null);
        LVIMSDK.sharedInstance().sendMessage(msg, null, new IMBridger.IMSendMessageListener() {
            @Override
            public void onIMSendMessageCallback(int ecode, String tid, IMMsg msg, Object o) {
                Map<String, Object> messageResultMap = convertMsgToMap(msg);
                messageResultMap.put("eCode", ecode);
                result.success(messageResultMap);
            }
        });

    }


    private void setIMToken(MethodCall call, Result result) {
        String userId = call.argument("userId");
        String imToken = call.argument("imToken");
        LVIMSDK.sharedInstance().setIMToken(userId, imToken);
    }

    private void requestDebugToken(MethodCall call, final Result result) {
        Log.d(TAG, "requestDebugToken， thread " + Thread.currentThread().getName());
        /**
         * 此方法仅在调试模式下有效，demo演示用。
         * IM token请通过自己应用的服务端向IM SDK服务器请求，再转发给客户端，然后调用setIMToken方法。
         */
        LVIMSDK.sharedInstance().requestDebugToken(new LVIMSDK.RequestDebugTokenListener() {
            @Override
            public void onSucceed(final String imToken) {
                Log.d(TAG, "获取IM  onSucceed: " + imToken + " thread " + Thread.currentThread().getName());
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        final Map<String, Object> map = new HashMap<>();
                        map.put("code", 0);
                        map.put("result", imToken);
                        result.success(map);
                    }
                });
            }

            @Override
            public void onFailed(final Exception e) {
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        final Map<String, Object> map = new HashMap<>();
                        map.put("code", -1);
                        map.put("result", e.toString());
                        result.success(map);
                    }
                });
                Log.d(TAG, "获取IM token失败， onFailure: " + e);
            }
        });
    }


    private void initSDK(final MethodCall call, final MethodChannel.Result result) {
        Log.d(TAG, "initSDK");
        String rtcAppID = call.argument("rtcAppID");
        String rtcAppKey = call.argument("rtcAppKey");
        String imAppID = call.argument("imAppID");
        String imAppKey = call.argument("imAppKey");
        int sdkType = call.argument("sdkType");
        boolean isDebugEnv = call.argument("isDebugEnv");
        boolean isInternationalEnv = call.argument("isInternationalEnv");
        mRtmEngine = LVRTMEngine.createEngine((Application) mContext.getApplicationContext(), rtcAppID, rtcAppKey, imAppID, imAppKey, sdkType, isDebugEnv, isInternationalEnv, new LinkVRTCEngine.IInitHandler() {
            @Override
            public void onInitResult(int resultCode) {
                result.success(resultCode);
            }
        });
        mRtmEngine.setIMAuthEventListener(imAuthEventListener);
        mRtmEngine.setGlobalReceiveMessageListener(new IMBridger.IMReceiveMessageListener() {
            @Override
            public boolean onIMReceiveMessageFilter(int cmdtype, int subtype, int diytype, int dataid, String fromid, String toid, String msgType, byte[] msgContent, int waitings, int packetSize, int waitLength, int bufferSize) {
                if (cmdtype == IMMsg.IM_CMDTYPE_ROOM){
                    LogUtils.d(TAG, "全局消息监听 ，过滤掉房间消息。 = " + cmdtype);
                    return true;
                }
                return false;
            }

            @Override
            public int onIMReceiveMessageHandler(String owner, IMMsg msg, int waitings, int packetSize, int waitLength, int bufferSize) {
                LogUtils.d(TAG, "onIMReceiveMessageHandler getMsgContent = " + msg.getMsgContent());
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        channel.invokeMethod("onIMMessageReceive", convertMsgToMap(msg));
                    }
                });
                return 0;
            }
        });
    }


    private void loginUser(final MethodCall call, final MethodChannel.Result result) {
        String userId = call.argument("userId");
        int resultCode = mRtmEngine.loginUser(userId);
        result.success(resultCode);
    }

    // 设置是否打开日志
    private void setLogOpen(MethodCall call) {
        boolean isOpen = call.argument("isOpen");
        mRtmEngine.setLogOpen(isOpen);
    }

    /**
     * 推流参数配置
     *
     * @param videoConfigLevel 推流参数预设值
     */
    public void setVideoConfig(int videoConfigLevel) {
//        mRtmEngine.setVideoConfig(videoConfigLevel);
    }


    // 发送私信消息
    private void sendPrivateMessage(MethodCall call, final Result result) {
        int sub = call.argument("sub");
        String tid = call.argument("tid");
        String type = call.argument("type");
        String content = call.argument("content");
        String  pushTitle = call.argument("pushTitle");
        String  pushBody = call.argument("pushBody");
        String  pushExtra = call.argument("pushExtra");
        String  pushClickAction = call.argument("pushClickAction");
        LVPushContent lvPushContent = new LVPushContent();
        lvPushContent.setTitle(pushTitle);
        lvPushContent.setBody(pushBody);
        lvPushContent.setExtra(pushExtra);
        lvPushContent.setClick_action(pushClickAction);
        mRtmEngine.sendPrivateMessage(convertSubTypeFromInt(sub), tid, type, content,lvPushContent, new IMBridger.IMSendMessageListener() {
            @Override
            public void onIMSendMessageCallback(int ecode, String tid, IMMsg msg, Object o) {
                Map<String, Object> messageResultMap = convertMsgToMap(msg);
                messageResultMap.put("eCode", ecode);
                result.success(messageResultMap);
            }
        });
    }


    private LVRTMEngine.MessageSubType convertSubTypeFromInt(int sub) {
        LVRTMEngine.MessageSubType type = IM_SUBTYPE_TEXT;
        switch (sub) {
            case 1:
                type = IM_SUBTYPE_IMAGE;
                break;
            case 2:
                type = IM_SUBTYPE_AUDIO;
                break;
            case 3:
                type = IM_SUBTYPE_VIDEO;
                break;
            default:
                type = IM_SUBTYPE_TEXT;
                break;
        }
        return type;
    }


    // 发送房间消息
    void sendRoomMessage(MethodCall call, final Result result) {
        String targetId = call.argument("targetId");
        String msgContent = call.argument("msgContent");
        String msgType = call.argument("msgType");
        LogUtils.d(TAG, "sendRoomMessage msgContent :" + msgContent);

        mRtmEngine.sendRoomMessage(targetId, msgContent, msgType, new IMBridger.IMSendMessageListener() {
            @Override
            public void onIMSendMessageCallback(int ecode, String tid, IMMsg msg, Object o) {
                Map<String, Object> messageResultMap = convertMsgToMap(msg);
                messageResultMap.put("eCode", ecode);
                result.success(messageResultMap);
            }
        });
    }

    // 将IMMsg对象的部分字段转成Map
    private Map<String, Object> convertMsgToMap(IMMsg msg) {
        final Map<String, Object> map = new HashMap<>();
        map.put("origin", msg.origin);
        map.put("cmdType", msg.cmdType);
        map.put("subType", msg.subType);
        map.put("fromID", msg.fromID);
        map.put("toID", msg.toID);
        map.put("msgContent", msg.msgContent);
        map.put("msgType", msg.extend1);
        map.put("extend2", msg.extend2);
        map.put("extend3", msg.extend3);
        map.put("extend4", msg.extend4);
        map.put("extend5", msg.extend5);
        map.put("extend6", msg.extend6);
        map.put("pushTitle", msg.pushTitle);
        map.put("cmsgid", msg.cmsgid);
        map.put("smsgid", msg.smsgid);
        map.put("dbid", msg.dbid);
        map.put("ctime", msg.ctime);
        map.put("stime", msg.stime);
        IMMsg.LVIM_MESSAGE_SEND_STATE send_state = msg.send_state;
        int value = 0;
        if (send_state != null) {
            value = send_state.value();
        }
        map.put("sendState", value);
        return map;
    }

    private void setBeautyLevel(final MethodCall call, final Result result) {
        int beauty = call.argument("beauty");
        double level_d = call.argument("level");
        BigDecimal decimal = new BigDecimal(level_d);
        float level = decimal.floatValue();
        switch (beauty) {
            case 0:
                mRtmEngine.setBeautyLevel(level);
                break;
            case 1:
                mRtmEngine.setBrightLevel(level);
                break;
            case 2:
                mRtmEngine.setToneLevel(level);
                break;
            default:
                break;
        }
    }

    private void switchCamera(final MethodCall call) {
        boolean isFrontCamera = call.argument("isFrontCamera");
        mRtmEngine.useFrontCamera(isFrontCamera);
    }

    private void setAVConfig(final MethodCall call, final Result result) {
        int config = call.argument("config");
        mRtmEngine.setVideoConfig(convertVideoConfigFromInt(config));

        Log.d(TAG, "setAVConfig config = " + config);
        result.success(true);
    }


    private LinkVRTCEngine.VideoConfigLevel convertVideoConfigFromInt(int config) {
        LinkVRTCEngine.VideoConfigLevel configLevel = VIDEO_CONFIG_360P;
        switch (config) {
            case 0:
                configLevel = VIDEO_CONFIG_180P;
                break;
            case 1:
                configLevel = VIDEO_CONFIG_270P;
                break;
            case 2:
                configLevel = VIDEO_CONFIG_360P;
                break;
            case 3:
                configLevel = VIDEO_CONFIG_540P;
                break;
            case 4:
                configLevel = VIDEO_CONFIG_720P;
                break;
            case 5:
                configLevel = VIDEO_CONFIG_1080P;
                break;
            default:
                break;
        }
        return configLevel;
    }


    private void setMuteOutput(final MethodCall call) {
        boolean isMute = call.argument("isMute");
        mRtmEngine.setMuteOutput(isMute);
    }


    /**
     * 打开摄像头数据
     */
    private void startCapture(final Result result) {
        if (mViewEventListener != null) {
            mViewEventListener.onStartCapture();
        }
        result.success(true);
    }

    /**
     * 关闭摄像头数据
     */
    private void stopCapture(final Result result) {
        if (mViewEventListener != null) {
            mViewEventListener.onStopCapture();
        }
        result.success(true);
    }


    private void isIMAuthed(final Result result) {
        result.success(mRtmEngine.isIMAuthed());
    }

    private void isIMLoginSucceed(final Result result) {
        result.success(mRtmEngine.isIMLoginSucceed());
    }

    private void setPlayQualityMonitor(final MethodCall call, final Result result) {
        Log.d(TAG, "setPlayQualityMonitor seconds = " + call.argument("seconds"));
//        mRtmEngine.setPlayQualityMonitorCycle((int) call.argument("seconds"));
        result.success(true);
    }

    private void stopPlayingStream(final MethodCall call, final Result result) {
        String streamId = call.argument("streamId");
        mRtmEngine.stopPlayingStream(streamId);
        result.success(true);
    }

    private void startPlayingStream(final MethodCall call, final Result result) {
        String streamId = call.argument("streamId");
        Log.d(TAG, "startPlayingStream streamId = " + streamId);
        boolean isZOrderMediaOverlay = call.argument("isZOrderMediaOverlay");
        mRtmEngine.startPlayingStream(streamId, null, isZOrderMediaOverlay);
        result.success(true);
    }

    private void stopPublishStream(final Result result) {
        mRtmEngine.stopPublishingStream();
        result.success(true);
    }

    private void logoutRoom(final Result result) {
        Log.d(TAG, "logout start");
        mRtmEngine.logoutRoom();
    }

    private void logoutUser(final Result result) {
        Log.d(TAG, "logoutUser start");
        LVIMSDK.sharedInstance().logout();
    }


    LVRTMEngine.IRoomEventHandler mRoomEventHandler = new LVRTMEngine.IRoomEventHandler() {
        @Override
        public void onRemoteStreamEnd(String userId) {
            LogUtils.d(TAG, "onRemoteStreamEnd user = " + userId);
            final Map<String, Object> map = new HashMap<>();
            map.put("userId", userId);
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onRemoteStreamEnd", map);
                }
            });
        }

        @Override
        public void onRemoteStreamAdd(String userId) {
            LogUtils.d(TAG, "onRemoteStreamAdd userId = " + userId);

            final Map<String, Object> map = new HashMap<>();
            map.put("userId", userId);

            Log.d(TAG, "onAddRemoter Android user_id = " + userId);
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onRemoteStreamAdd", map);
                }
            });
        }

        @Override
        public void onRoomDisconnect(int errorCode, String roomID) {
            LogUtils.d(TAG, "onRoomDisconnect errorCode = " + errorCode);

            final Map<String, Object> map = new HashMap<>();
            map.put("errorCode", errorCode);
            map.put("roomID", roomID);
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onRoomDisconnect", map);
                }
            });

        }

        @Override
        public void onRoomConnected(String roomID) {
            LogUtils.d(TAG, "onRoomConnected roomID = " + roomID);
            final Map<String, Object> map = new HashMap<>();
            map.put("roomID", roomID);
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onRoomConnected", map);
                }
            });
        }

        @Override
        public void onRemoteQualityUpdate(String streamID, VideoQuality quality) {
            final Map<String, Object> map = new HashMap<>();
            map.put("streamID", streamID);
            map.put("videoSentPackets", quality.videoSentPackets);
            map.put("videoSentKBytes", quality.videoSentKBytes);
            map.put("videoFps", quality.videoFps);
            map.put("videoLostPackets", quality.videoLostPackets);
            map.put("videoBitrateBps", quality.videoBitrateBps);
            map.put("videoRtt", quality.videoRtt);
            map.put("videoLostPercent", quality.videoLostPercent);
            map.put("frameWidth", quality.frameWidth);
            map.put("frameHeight", quality.frameHeight);

            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onRemoteQualityUpdate", map);
                }
            });
        }

        @Override
        public void onPublisherQualityUpdate(String streamID, VideoQuality quality) {
            final Map<String, Object> map = new HashMap<>();
            map.put("videoSentPackets", quality.videoSentPackets);
            map.put("videoSentKBytes", quality.videoSentKBytes);
            map.put("videoFps", quality.videoFps);
            map.put("videoLostPackets", quality.videoLostPackets);
            map.put("videoBitrateBps", quality.videoBitrateBps);
            map.put("videoRtt", quality.videoRtt);
            map.put("videoLostPercent", quality.videoLostPercent);
            map.put("frameWidth", quality.frameWidth);
            map.put("frameHeight", quality.frameHeight);
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onPublisherQualityUpdate", map);
                }
            });
        }

        @Override
        public void onVideoSizeChanged(String streamID, int width, int height) {
            final Map<String, Object> map = new HashMap<>();
            map.put("streamID", streamID);
            map.put("width", width);
            map.put("height", height);
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onVideoSizeChanged", map);
                }
            });
        }

        @Override
        public void onRoomMessageReceive(IMMsg msg) {
            LogUtils.d(TAG, "onRoomMessageReceive getMsgContent = " + msg.getMsgContent());
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onRoomMessageReceive", convertMsgToMap(msg));
                }
            });
        }

        @Override
        public void onPublishStateUpdate(int state) {
            LogUtils.d(TAG, "onPublishStateUpdate state = " + state);
            final Map<String, Object> map = new HashMap<>();
            map.put("state", state);
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onPublishStateUpdate", map);
                }
            });
        }

        @Override
        public void onPlayStateUpdate(int state, String userId) {
            LogUtils.d(TAG, "onPlayStateUpdate state = " + state + "  userId: " + userId);
            final Map<String, Object> map = new HashMap<>();
            map.put("state", state);
            map.put("streamId", userId);
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onPlayStateUpdate", map);
                }
            });
        }
    };


    IMBridger.IMModuleEventListener imAuthEventListener = new IMBridger.IMModuleEventListener() {
        @Override
        public void onQueryIMToken() { // 请在此回调中请求并设置IM token。此回调会频繁调用，请注意加逻辑控制避免频繁请求。示例如下。
            final Map<String, Object> map = new HashMap<>();
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onQueryIMToken", map);
                }
            });

        }

        @Override
        public void onIMAuthFailed(String uid, String token, int ecode, int rcode, boolean isTokenExpired) {
            // IM token校验失败会回调此方法。同时也回调onQueryIMToken().
            Log.d(TAG, "IM token校验失败 错误码 = " + ecode);
            final Map<String, Object> map = new HashMap<>();
            map.put("uid", uid);
            map.put("token", token);
            map.put("ecode", ecode);
            map.put("rcode", rcode);
            map.put("isTokenExpired", isTokenExpired);
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onIMAuthFailed", map);
                }
            });
        }

        @Override
        public void onIMAuthSucceed(String uid, String token, long unReadMsgSize) {
            // IM校验token通过
            Log.d(TAG, "IM token校验通过 uid = " + uid + "    token =" + token);
            final Map<String, Object> map = new HashMap<>();
            map.put("uid", uid);
            map.put("token", token);
            map.put("unReadMsgSize", unReadMsgSize);
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onIMAuthSucceed", map);
                }
            });
        }

        @Override
        public void onIMTokenExpired(String uid, String token) {
            // IM token过期会回调此方法。同时也回调onQueryIMToken().
            Log.d(TAG, "IM token过期 uid = " + uid + "    token =" + token);
            final Map<String, Object> map = new HashMap<>();
            map.put("token", token);
            map.put("uid", uid);
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onIMTokenExpired", map);
                }
            });
        }
    };


    private void unInitSDK(final MethodCall call, final MethodChannel.Result result) {
        if (mRtmEngine != null)
            mRtmEngine.unInitSDK();
        mRtmEngine = null;
    }


    private void loginRoom(final MethodCall call, final MethodChannel.Result result) {
        String userId = (String) call.argument("userId");
        Log.d(TAG, "loginRoom " + userId);
        mRtmEngine.setRoomEventHandler(mRoomEventHandler);
        mRtmEngine.loginRoom(userId, (String) call.argument("roomId"), (boolean) call.argument("isHost"), (boolean) call.argument("isAudioOnly"));
    }

    private void setPublishQualityMonitorCycle(final MethodCall call, final MethodChannel.Result result) {
        Log.d(TAG, "setPublishQualityMonitorCycle");
//        mRtmEngine.setPublishQualityMonitorCycle((int) call.argument("seconds"));
        result.success(true);
    }

    private void startPublishing(final MethodCall call, final MethodChannel.Result result) {
        Log.d(TAG, "startPublishing");
        mRtmEngine.startPublishing();
        result.success(true);
    }


    @Override
    public void onDetachedFromEngine(FlutterPluginBinding binding) {
        LogUtils.d(TAG, "onDetachedFromEngine");
        channel.setMethodCallHandler(null);
    }


    static void setEventListener(ViewEventListener listener) {
        mViewEventListener = listener;
    }


    public interface ViewEventListener {
        void onStartCapture();

        void onStopCapture();
    }

    private void uploadPushToken(final MethodCall call, final MethodChannel.Result result) {
        String token = call.argument("token");
        if (TextUtils.isEmpty(LVIMSDK.sharedInstance().getUserID())) {
            Map<String, Object> map = new HashMap<>();
            map.put("eCode", -1);
            map.put("msg", "please login IM first");
            result.success(map);
        }
        LVIMSDK.sharedInstance().uploadPushToken(token, new IMBridger.IMUploadPushTokenListener() {
            @Override
            public void onIMUploadPushTokenCallback(int eCode, String msg) {
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        Map<String, Object> map = new HashMap<>();
                        map.put("eCode", eCode);
                        map.put("msg", msg);
                        result.success(map);
                    }
                });
            }
        });
    }

}
