//
//  LVRTMEngine.h
//  LinkV_Zego_iOS
//
//  Created by Wing on 2020/8/5.
//

#import <Foundation/Foundation.h>
#import "LinkVRTCEngine.h"
#import <LVIMLib/LVIMLib.h>

static int ERROR_BUILD_MSG = 1001; // 构建消息失败

@protocol RoomCallbackProtocl<RTCCallbackDelegate>

/// 收到房间消息
/// 如果返回非0, 则这条消息会在下次冷启动或再次检测到时继续调用本方法
- (int)onRoomMessageReceive:(LVIMMessage *)msg;

@end

@interface LVRTMEngine : NSObject

+ (instancetype)shared;

+ (instancetype)createEngine:(SDKType)type
                    rtcAppId:(NSString *)rtcAppId
                   rtcAppKey:(NSString *)rtcAppKey
                     imAppId:(NSString *)imAppId
                    imAppKey:(NSString *)imAppKey
                   isTestEnv:(BOOL)isTestEnv
          isInternationalEnv:(BOOL)isInternationalEnv
                    delegate:(id<LVIMModuleEventDelegate, LVIMReceiveMessageDelegate, RoomCallbackProtocl>)delegate;

- (void)setLogOpen:(BOOL)isOpen;

#pragma mark - RTC
// 推流参数配置
- (void)setVideoConfig:(VideoConfigLevel)configLevel;
- (void)loginRoom:(NSString *)uid roomId:(NSString *)roomId isHost:(BOOL)isHost isAudioOnly:(BOOL)isAudioOnly;
- (void)logoutRoom;
- (void)startPreview:(UIView *)view;
- (void)startPublishing;
- (void)startPlayingStream:(NSString *)uid inView:(UIView *)view;
- (void)stopPlayingStream:(NSString *)uid;
// 转换前后置摄像头
- (void)useFrontCamera:(BOOL)isFront;
// 获取连麦SDK的名称
- (NSString *)getSolutionName;
// 获取支持连麦数
- (int)getSupportLine;
// 获取SDK类型 3为自研，4为zego
- (SDKType)getSDKType;
// 设置是否静音推流
- (void)enableMic:(BOOL)isMute;
// 释放引擎
- (void)unInitSDK;
- (int)getMicVolume;
/// sdk版本号
- (NSString *)sdkVersion;

/// 设置推流时数据统计信息回调的频率，回调方法请看 OnPublishQualityUpdate 回调
/// @param seconds 回调频率（1 / sec）
- (void)setPublishQualityMonitorCycle:(int)seconds;
/// 设置拉流时数据统计信息回调的频率，回调方法请看 OnPlayQualityUpate 回调
/// @param seconds 回调频率（1 / sec）
- (void)setPlayQualityMonitorCycle:(int)seconds;


/// 设置是否允许 SDK 自动根据设备的方向调整视频的输出方向
/// @param enable 打开和关闭（默认是 YES）
-(void)enableVideoAutoRotation:(BOOL)enable;

/// 停止发布音视频数据
- (void)stopPublishStream;

/**
 * 设置美颜滤镜级别
 * @param beautyLevel 滤镜美颜级别 取值范围[0,1]，设置-1关闭
 */
- (void)setBeautyLevel:(CGFloat)beautyLevel;
/**
 * 设置滤镜明亮度
 * @param brightLevel 滤镜明亮度 取值范围[0,1]，设置-1关闭
 */
- (void)setBrightLevel:(CGFloat)brightLevel;
/**
 * 设置滤镜饱和度
 * @param toneLevel 滤镜饱和度 取值范围[0,1]，设置-1关闭
 */
- (void)setToneLevel:(CGFloat)toneLevel;

/// 开始采集视频流
- (void)startCapture;
/// 停止采集视频流
- (void)stopCapture;

#pragma mark - IM
// 登录IM
- (int)loginIM:(NSString *)uid;

// 设置IM鉴权监听
- (void)setIMAuthEventDelegate:(id<LVIMModuleEventDelegate>)delegate;

// 设置全局消息监听
- (void)setIMGlobalReceiveMessageDelegate:(id<LVIMReceiveMessageDelegate>)delegate;

// 设置房间消息监听
- (void)setRoomMsgDelegate:(id<RoomCallbackProtocl>)delegate;

/**
@brief 构建私信请求
@param subType 内容类型(见LVIM_MESSAGE_SUBTYPE)
@param tid 接收者ID(用户ID)
@param type 消息类型别名(类型唯一)
@param content 消息内容
**/
- (void)sendPrivateMessage:(LVIM_MESSAGE_SUBTYPE)subType
                       tid:(NSString *)tid
                      type:(NSString *)type
                   content:(NSString *)content
                  complete:(LVIM_SEND_MESSAGE_CALLBACK)complete;

/**
@brief 构建私信请求
@param subType 内容类型(见LVIM_MESSAGE_SUBTYPE)
@param tid 接收者ID(用户ID)
@param type 消息类型别名(类型唯一)
@param content 消息内容
@param pushTitle 推送标题
@param pushContent 推送内容
@param extend3 扩展字段(字符串:无特殊情况不要使用)
@param extend4 扩展字段(二进制:无特殊情况不要使用)
@param targetAppID 扩展字段5(目标应用ID)
@param targetAppUID 扩展字段6(目标应用UID)
**/
- (void)sendPrivateMessage:(LVIM_MESSAGE_SUBTYPE)subType
                       tid:(NSString *)tid
                      type:(NSString *)type
                   content:(NSString *)content
                 pushTitle:(NSString *)pushTitle
               pushContent:(NSString *)pushContent
                pushExtra:(NSString *)pushExtra
                   extend3:(NSString *)extend3
                   extend4:(NSData *)extend4
               targetAppID:(NSString *)targetAppID
              targetAppUID:(NSString *)targetAppUID
                  complete:(LVIM_SEND_MESSAGE_CALLBACK)complete;

/**
 * 发送房间消息
 * @param roomId   房间Id
 * @param content 消息内容
 * @param complete   发送结果监听
 */
- (void)sendRoomMessage:(NSString *)roomId content:(NSString *)content complete:(LVIM_SEND_MESSAGE_CALLBACK)complete;

/**
 * 发送房间消息
 * @param roomId   房间Id
 * @param content 消息内容
 * @param msgType 消息的自定义扩展类型，可作为消息标识，但不能传空或者空串
 * @param complete   发送结果监听
 */
- (void)sendRoomMessage:(NSString *)roomId content:(NSString *)content msgType:(NSString *)msgType complete:(LVIM_SEND_MESSAGE_CALLBACK)complete;

@end

