import 'dart:typed_data';
import 'dart:convert' show utf8;

enum LVCameraPosition {
  back, // 后置摄像头
  front, // 前置摄像头
}

enum LVAVConfig {
  PROFILE_180P,
  PROFILE_270P,
  PROFILE_360P,
  PROFILE_540P,
  PROFILE_720P,
}

enum LVBeauty {
  Beauty, // 美颜级别
  Bright, // 明亮度
  Tone // 饱和度
}

enum LVIMMsgState {
  sending,          // 发送中
  failed,           // 发送失败
  sent_succ_unread, // 发送成功，接收者未读
  sent_succ_read,   // 发送成功，接收者已读
  receive_unread,   // 接收成功，自己未读
  receive_read,     // 接收成功，自己已读
}

enum MessageSubType {
  //文本类协议
  IM_SUBTYPE_TEXT,
//图片类协议
  IM_SUBTYPE_IMAGE,
//音频类协议
  IM_SUBTYPE_AUDIO,
//视频类协议
  IM_SUBTYPE_VIDEO
}

class LVUser {
  String userId;
  List<String> pullUrls; // 拉流url集合
}

class LinkvLoginRoomResult {
  int code;
  List<LVUser> users;
}

class LVVideoStatistic {
  // 音视频数据统计信息
  int videoSentPackets; // 视频总发送包数量
  int videoSentKBytes; // 视频总发送字节数
  int videoFps; // 视频输入帧率（视频帧率）
  int videoLostPackets; // 视频丢包总数
  int videoBitrateBps; // 视频码率
  int videoRtt; // 音频包 RTT，单位 ms
  int videoLostPercent; // 累计视频丢包百分比
  int frameWidth; // 推流：编码视频宽，拉流：输入视频帧宽
  int frameHeight; // 推流：编码视频高，拉流：输入视频帧高

  static LVVideoStatistic convertFromMap(dynamic args) {
    LVVideoStatistic v = LVVideoStatistic();
    v.videoSentPackets = args["videoSentPackets"];
    v.videoSentKBytes = args["videoSentKBytes"];
    v.videoFps = args["videoFps"];
    v.videoLostPackets = args["videoLostPackets"];
    v.videoBitrateBps = args["videoBitrateBps"];
    v.videoRtt = args["videoRtt"];
    v.videoLostPercent = args["videoLostPercent"];
    v.frameWidth = args["frameWidth"];
    v.frameHeight = args["frameHeight"];

    return v;
  }
}

class LVIMMsg {
  //消息来源
  int origin;

  //消息类型
  int cmdType = -1;

  //内容类型
  int subType = -1;

  //来源ID(发送者ID)
  String fromID;

  //目标ID(接受者ID)
  String toID;

  //消息内容
  String msgContent;

  //扩展内容1(一般为msgType)
  String msgType;

  //扩展内容2
  String extend2;

  //扩展内容3
  String extend3;

  //扩展内容4(二进制)
  var extend4;

  //扩展内容5
  String extend5;

  //扩展内容6
  String extend6;

  //推送标题
  String pushTitle;

  //客户端消息ID
  int cmsgid;

  //服务端消息ID
  int smsgid;

  //本地数据库ID
  int dbid;

  //客户端时间戳
  int ctime;

  //服务端时间戳
  int stime;

  // 消息发送状态
  LVIMMsgState sendState;

  static LVIMMsg convertFromMap(dynamic args) {
    LVIMMsg msg = LVIMMsg();
    //消息来源
    msg.origin = args["origin"];
    //消息类型
    msg.cmdType = args["cmdType"];
    //内容类型
    msg.subType = args["subType"];
    //来源ID(发送者ID)
    msg.fromID = args["fromID"];
    //目标ID(接受者ID)
    msg.toID = args["toID"];
    //消息内容
    msg.msgContent = convertByteArrayToString(args["msgContent"]);
    //扩展内容1(一般为msgType)
    msg.msgType = convertByteArrayToString(args["msgType"]);
    //扩展内容2
    msg.extend2 = convertByteArrayToString(args["extend2"]);
    //扩展内容3
    msg.extend3 = convertByteArrayToString(args["extend3"]);
    //扩展内容4(二进制)
    msg.extend4 = args["extend4"];
    //扩展内容5
    msg.extend5 = convertByteArrayToString(args["extend5"]);
    //扩展内容6
    msg.extend6 = convertByteArrayToString(args["extend6"]);
    //推送标题
    msg.pushTitle = convertByteArrayToString(args["pushTitle"]);
    msg.cmsgid = args["cmsgid"];
    msg.smsgid = args["smsgid"];
    msg.dbid = args["dbid"];
    msg.ctime = args["ctime"];
    msg.stime = args["stime"];
    if(args["sendState"] != null){
      msg.sendState = LVIMMsgState.values[args["sendState"]];
    }

    return msg;
  }

  // 字节数组转String
  static String convertByteArrayToString(arg) {
    if (arg == null) {
      return null;
    }
    return utf8.decode(arg);
  }
}

/// 发送消息的结果
class SendMessageResult {
  int eCode;
  LVIMMsg msg;

  static SendMessageResult convertFromMap(dynamic args) {
    SendMessageResult sendMessageResult = SendMessageResult();
    sendMessageResult.msg = LVIMMsg.convertFromMap(args);
    sendMessageResult.eCode = args["eCode"];
    return sendMessageResult;
  }
}

/// 上传push token的返回结果
/// eCode 错误码，0为成功，非0为失败。
/// msg  错误信息
class UploadPushTokenResult {
  int eCode;
  String msg;

  static UploadPushTokenResult convertFromMap(dynamic args) {
    UploadPushTokenResult result = UploadPushTokenResult();
    result.eCode = args["eCode"];
    result.msg = args["msg"];
    return result;
  }
}

/// 推送内容
/// title 推送标题
/// body  推送内容
/// extra 推送自定义负载数据
class LVPushContent {
  String title;
  String body;
  String extra;
  String click_action;
}

/// 会话
class LVIMSession {
  LVIMMsg lastMsg;
  int unreadCount;
}
